# -*- coding: utf-8 -*-
""" Commands for building and pushing the app image.

This is mainly for web apps that are deployed with docker.
"""
from __future__ import absolute_import, unicode_literals
from . import cli, click


@cli.group('docker')
def docker_cli():
    """ Commands for building the project docker images. """
    pass


@docker_cli.command('build')
def build_images():
    """ Build and tag a docker image for the project.

    It requires a DOCKER_IMAGES conf variable to be set and contain the
    definitions for all the images built by the project. If DOCKER_REGISTRY is
    also set, then the resulting images will a have name of {registry}/{image}.
    This will create two tags for each image: {name}:{version} and
    {name}:latest where {name} is the image name as specified in DOCKER_IMAGES
    entry and {version} is the current project version (as shown by
    `peltak version`).

    Here is an example DOCKER_IMAGES:

        conf.init({
            # ...
            'DOCKER_IMAGES': [
                {
                    'name': 'myapp',
                    'path': '.',
                    'file': './ops/Dockerfile'
                }
            ]
        })

    """
    from peltak.core import conf
    from peltak.core import docker

    registry = conf.get('DOCKER_REGISTRY')
    docker_images = conf.get('DOCKER_IMAGES', [])

    for image in docker_images:
        docker.build_image(registry, image)


@docker_cli.command('push')
def push_images():
    """ Push project docker images to the registry.

    This command requires both DOCKER_IMAGES and DOCKER_REGISTRY conf variables
    to be set. This will push the images built by `peltak docker build` to the
    specified registry. For the details of how the names and tags are generated,
    see `peltak docker build --help`.
    """
    import sys
    from peltak.core import conf
    from peltak.core import docker
    from peltak.core import log

    registry = conf.get('DOCKER_REGISTRY')
    docker_images = conf.get('DOCKER_IMAGES', [])

    if registry is None:
        log.err("You must define DOCKER_REGISTRY conf variable to push images")
        sys.exit(-1)

    for image in docker_images:
        docker.push_image(registry, image)


@docker_cli.command('list')
@click.option(
    '-p', '--password', 'registry_pass',
    type=str,
    prompt='Password',
    hide_input=True,
    help='Registry password'
)
def docker_list(registry_pass):
    """ List docker images and their tags on the remote registry.

    This command requires the DOCKER_REGISTRY conf variable to be set. You can
    also hard-code the username with the DOCKER_REGISTRY_USER conf variable. You
    will be asked for the password every time though for security reasons.
    """
    import sys
    from peltak.core import conf
    from peltak.core import docker
    from peltak.core import log
    from peltak.core import shell

    registry = conf.get('DOCKER_REGISTRY', None)

    if registry is None:
        log.err("You must define DOCKER_REGISTRY conf variable to list images")
        sys.exit(-1)

    registry_user = conf.get('DOCKER_REGISTRY_USER', None)

    if registry_user is None:
        registry_user = click.prompt("Username")

    rc = docker.RegistryClient(registry, registry_user, registry_pass)
    images = {x: rc.list_tags(x) for x in rc.list_images()}

    shell.cprint("<32>Images in <34>{} <32>registry:", registry)
    for image, tags in images.items():
        shell.cprint('  <92>{}', image)
        for tag in tags:
            shell.cprint('      <90>{}:<35>{}', image, tag)



# -*- coding: utf-8 -*-
"""
git helpers.
"""
from __future__ import absolute_import, unicode_literals
from . import cli, click


@cli.group('git')
def git_group():
    """ Git related commands """
    pass


@git_group.command('add-hooks')
def add_hooks():
    """ Setup project git hooks.

    This will run all the checks before pushing to avoid waiting for the CI
    fail.
    """
    import os
    from peltak.core import conf
    from peltak.core import log

    log.info("Adding pre-commit hook")
    with open(conf.proj_path('.git/hooks/pre-commit'), 'w') as fp:
        fp.write('\n'.join([
            '#!/bin/bash',
            'PATH="/opt/local/libexec/gnubin:$PATH"',
            (
                'REPO_PATH=$(dirname "$(dirname "$(dirname '
                '"$(readlink -fm "$0")")")")'
            ),
            '',
            'source "$REPO_PATH/env/bin/activate"',
            '',
            'peltak lint',
        ]))
        fp.write('\n')

    log.info("Adding pre-push hook")
    with open(conf.proj_path('.git/hooks/pre-push'), 'w') as fp:
        fp.write('\n'.join([
            '#!/bin/bash',
            'PATH="/opt/local/libexec/gnubin:$PATH"',
            (
                'REPO_PATH=$(dirname "$(dirname "$(dirname '
                '"$(readlink -fm "$0")")")")'
            ),
            '',
            'source "$REPO_PATH/env/bin/activate"',
            '',
            'peltak test --allow-empty',
        ]))
        fp.write('\n')

    log.info("Making hooks executable")
    os.chmod(conf.proj_path('.git/hooks/pre-commit'), 0o755)
    os.chmod(conf.proj_path('.git/hooks/pre-push'), 0o755)


@git_group.command()
def push():
    """ Push the current branch and set to track remote. """
    from peltak.core import git
    from peltak.core import shell

    branch = git.current_branch()
    shell.run('git push -u origin {}'.format(branch))


@git_group.command()
@click.argument('target', required=False)
def merged(target=None):
    """ Checkout develop, pull and delete merged branches.

    This is to ease the repetitive cleanup of each merged branch.
    """
    from fnmatch import fnmatch
    from peltak.core import conf
    from peltak.core import git
    from peltak.core import log
    from peltak.core import shell

    main_branch = conf.get('MAIN_BRANCH', 'develop')
    master_branch = conf.get('MASTER_BRANCH', 'master')
    protected_branches = conf.get('PROTECTED_BRANCHES', ('master', 'develop'))
    release_branch_pattern = conf.get('RELEASE_BRANCH_PATTERN', 'release/*')
    branch = git.current_branch()

    if target is None:
        if fnmatch(branch, release_branch_pattern):
            target = master_branch
        else:
            target = main_branch

    try:
        shell.run('git rev-parse --verify {}'.format(branch))
    except IOError:
        log.err("Branch '{}' does not exist".format(branch))

    log.info("Checking out <33>{}".format(target))
    shell.run('git checkout {}'.format(target))

    log.info("Pulling latest changes")
    shell.run('git pull origin {}'.format(target))

    if branch not in protected_branches:
        log.info("Deleting branch <33>{}".format(branch))
        shell.run('git branch -d {}'.format(branch))

    log.info("Pruning")
    shell.run('git fetch --prune origin')

    log.info("Checking out <33>{}<32> branch".format(target))
    shell.run('git checkout {}'.format(target))



# -*- coding: utf-8 -*-
"""
Various commands that do not fit another category but there's not enough of them
to justify a separate module.
"""
from __future__ import absolute_import, unicode_literals, print_function
from . import cli, click


@cli.command()
@click.option('--recreate', is_flag=True)
@click.option('--no_index', is_flag=True)
def docs(recreate=False, no_index=False):
    """ Build project documentation. """
    import os.path
    import shutil
    from peltak.core import conf
    from peltak.core import log
    from peltak.core import shell

    build_dir = conf.get_path('BUILD_DIR', '.build')
    doc_src_path = conf.get_path('DOC_SRC_PATHS', 'docs')
    refdoc_paths = conf.get('REFDOC_PATHS', [])

    doc_out_path = os.path.join(doc_src_path, 'html')
    doc_ref_path = os.path.join(doc_src_path, 'ref')
    doc_assets_path = os.path.join(doc_src_path, 'assets')
    doc_build_path = os.path.join(build_dir, 'docs')

    log.info('Ensuring assets directory <94>{}<32> exists', doc_assets_path)
    if not os.path.exists(doc_assets_path):
        os.makedirs(doc_assets_path)

    if recreate and os.path.exists(doc_out_path):
        log.info("<91>Deleting <94>{}".format(doc_out_path))
        shutil.rmtree(doc_out_path)

    if refdoc_paths:
        _gen_ref_docs(doc_ref_path, no_index)
    else:
        log.err('Not generating any reference documentation - '
                'No REFDOC_PATHS specified in config')

    with conf.within_proj_dir(doc_src_path):
        log.info('Building docs with <35>sphinx')
        shell.run('sphinx-build -b html -d {build} {docs} {out}'.format(
            build=doc_build_path,
            docs=doc_src_path,
            out=doc_out_path,
        ))


def _gen_ref_docs(ref_path, no_index=False):
    import os.path
    import shutil
    import sys
    from peltak.core import conf
    from peltak.core import log
    try:
        from refdoc import generate_docs as _generate_docs
    except ImportError:
        print("You need to install sphinx-refdoc to use the 'docs' command",
              file=sys.stderr)
        sys.exit(-1)

    refdoc_paths = conf.get('REFDOC_PATHS', [])

    log.info('Removing previously generated reference documentation')
    if os.path.exists(ref_path):
        shutil.rmtree(ref_path)

    os.makedirs(ref_path)

    log.info('Generating reference documentation')
    args = {'out_dir': ref_path}

    if no_index:
        args['no_index'] = True

    pkg_paths = [conf.proj_path(p) for p in refdoc_paths]

    _generate_docs(pkg_paths, **args)



# -*- coding: utf-8 -*-
"""
Testing commands
"""
from __future__ import absolute_import, unicode_literals
from . import cli, click


@cli.command()
@click.option('--no-sugar', is_flag=True)
@click.option('--type', 'tests_type', type=str, default='default')
@click.option('-v', '--verbose', count=True)
@click.option('--junit', is_flag=True)
@click.option('--no-locals', is_flag=True)
@click.option('--no-coverage', is_flag=True)
@click.option('--allow-empty', is_flag=True)
@click.option('--plugins', type=str, default='')
def test(
        no_sugar, tests_type, verbose, junit, no_locals,
        no_coverage, plugins, allow_empty
):
    """ Run all tests against the current python version. """
    import sys
    import os.path
    from peltak.core import conf
    from peltak.core import fs
    from peltak.core import shell

    build_dir = conf.get_path('BUILD_DIR', '.build')

    pytest_cfg_path = conf.get_path('PYTEST_CFG_PATH', 'ops/tools/pytest.ini')
    test_types = conf.get('TEST_TYPES', {})

    coverage_out_path = os.path.join(build_dir, 'coverage')
    coverage_cfg_path = conf.get_path('COVERAGE_CFG_PATH',
                                      'ops/tools/coverage.ini')

    django_settings = conf.get('DJANGO_SETTINGS', None)
    django_test_settings = conf.get('DJANGO_TEST_SETTINGS', None)

    src_dir = conf.get_path('SRC_DIR')
    src_path = conf.get_path('SRC_PATH')
    plugins = plugins.split(',')
    args = []

    if not no_coverage:
        args += [
            '--durations=3',
            '--cov-config={}'.format(coverage_cfg_path),
            '--cov={}'.format(src_path),
            '--cov-report=term:skip-covered',
            '--cov-report=html:{}'.format(coverage_out_path),
        ]

    if junit:
        args += ['--junitxml={}/test-results.xml'.format('.build')]

    if '-django' not in plugins:
        if django_test_settings is not None:
            args += ['--ds {}'.format(django_test_settings)]
        elif django_settings is not None:
            args += ['--ds {}'.format(django_settings)]

    if no_sugar:
        args += ['-p no:sugar']

    if verbose >= 1:
        args += ['-v']
    if verbose >= 2:
        args += ['--full-trace']

    if not no_locals:
        args += ['-l']

    if plugins:
        for plug_name in plugins:
            if not plug_name.strip():
                continue

            if plug_name.startswith('-'):
                args += ['-p no:{}'.format(plug_name[1:])]
            else:
                args += ['-p {}'.format(plug_name)]

    test_config = {'paths': src_path}
    if tests_type is not None:
        test_config = test_types.get(tests_type)
        mark = test_config.get('mark')

        if mark:
            args += ['-m "{}"'.format(mark)]

    test_paths = test_config['paths'] or []
    test_paths = [conf.proj_path(p) for p in test_paths]
    result = shell.run(
        'pytest -c {conf} {args} {paths}'.format(
            conf=pytest_cfg_path,
            args=' '.join(args),
            paths=fs.surround_paths_with_quotes(test_paths)
        ),
        env={'PYTHONPATH': src_dir},
        exit_on_error=False
    )

    if result.failed and not (allow_empty is True and result.return_code == 5):
        sys.exit(result.return_code)



# -*- coding: utf-8 -*-
""" Commands for managing the managed project version. """
from __future__ import absolute_import, unicode_literals
from . import cli, click


@cli.group('version', invoke_without_command=True)
@click.option('--porcelain', is_flag=True)
@click.pass_context
def ver(ctx, porcelain):
    """ Show project version. Has subcommands. """
    if not ctx.invoked_subcommand:
        _show_version(porcelain)


@ver.command('show')
@click.option('--porcelain', is_flag=True)
def show(porcelain):
    """ Deprecated. Use ``peltak version``. """
    _show_version(porcelain)


@ver.command('bump')
@click.argument(
    'component',
    type=click.Choice(['major', 'minor', 'patch']),
    required=False,
    default='patch'
)
@click.option('--exact', type=str)
def bump_version(component='patch', exact=None):
    """ Bump current project version without committing anything.

    No tags are created either.
    """
    from peltak.core import log
    from peltak.core import versioning

    old_ver, new_ver = versioning.bump(component, exact)

    log.info("Bumping package version")
    log.info("  old version: <35>{}".format(old_ver))
    log.info("  new version: <35>{}".format(new_ver))


def _show_version(porcelain):
    from peltak.core import log
    from peltak.core import versioning
    current = versioning.current()

    if porcelain:
        print(current)
    else:
        log.info("Version: <35>{}".format(current))



# -*- coding: utf-8 -*-
"""
Various commands that do not fit another category but there's not enough of them
to justify a separate module.
"""
from __future__ import absolute_import, unicode_literals
from . import cli


@cli.command()
def clean():
    """ Remove temporary files like python cache, swap files, etc. """
    import os
    from peltak.core import conf
    from peltak.core import fs

    cwd = os.getcwd()
    clean_patterns = conf.get('CLEAN_PATTERNS', [
        '__pycache__',
        '*.py[cod]',
        '.swp',
    ])

    os.chdir(conf.proj_path('.'))

    for pattern in clean_patterns:
        fs.rm_glob(pattern)

    os.chdir(cwd)


@cli.command()
def init():
    """ Create new peltak config file in the current directory """
    with open('pelconf.py', 'w') as fp:
        fp.write('''# -*- coding: utf-8 -*-
""" peltak configuration file.

See `peltak <https://github.com/novopl/peltak>`_ for more information.
"""
from __future__ import absolute_import

# Configure the build
from peltak.core import conf

conf.init({
})

# Import all commands
#from peltak.commands import version
''')
